#include <LedControl.h>
#include <stdio.h>
#include <string.h>

/* 
 * Create a new LedControl. 
 * We use pins 12,11 and 10 for the SPI interface
 * With our hardware we have connected pin 12 to the DATA IN-pin (1) of the first MAX7221
 * pin 11 is connected to the CLK-pin(13) of the first MAX7221
 * pin 10 is connected to the LOAD-pin(12) of the first MAX7221	 	
 * We will only have a single MAX7221 attached to the arduino 
 */
LedControl lc1=LedControl(12,11,10,1); 



void setup() {
  /* Set max intensity */
  lc1.setIntensity(0,15);
  /* Set all signals to 'dark' (no lights on). */
  lc1.clearDisplay(0);
  /* Wake up display. */
  lc1.shutdown(0,false);
  /* Announce ourself to the host */
  Serial.begin(115200);
  Serial.println("Signal Driver Max72XX 0.0");
  Serial.print("\n>>");
  Serial.flush();
}

/* Signal Aspects */
#define R_R B00001001 /* Red over Red (Stop) */
#define R_Y B00001010 /* Red over Yellow (Approach Limited) */
#define R_G B00001100 /* Red over Green (Slow Clear) */
#define Y_R B00010001 /* Yellow over Red (Approach) */
#define G_R B00100001 /* Green over red (Clear) */
#define DARK B00000000 /* Dark (all lights off) */

int GetAspectBits(const char *aspectname) {
  /** Test for each signal aspect string and when a match
    * Occurs, return the corresponding bit pattern. */
  if (strcasecmp("R_R",aspectname) == 0) return R_R;
  else if (strcasecmp("R_Y",aspectname) == 0) return R_Y;
  else if (strcasecmp("R_G",aspectname) == 0) return R_G;
  else if (strcasecmp("Y_R",aspectname) == 0) return Y_R;
  else if (strcasecmp("G_R",aspectname) == 0) return G_R;
  else if (strcasecmp("DARK",aspectname) == 0) return DARK;
  else return -1;
}
  
void loop() {
  char buffer[256]; /* Command line buffer. */
  int  len;         /* Line length. */

  /* If there is serial data available... */
  if (Serial.available() > 0) {
    /* Read a line from the serial port (USB connection
       from the host computer. */
    len = Serial.readBytesUntil('\r',buffer,sizeof(buffer)-1);
    if (len <= 1) {
      /* Reissue command prompt. */
      Serial.print("\n>>");
      Serial.flush();
      return;
    }
    buffer[len] = '\0';
    switch (toupper(buffer[0])) {
      case 'D': /* Clear all signals to Dark. */
        lc1.clearDisplay(0);
        break;
      case 'S': /* Set one signal. */
        {
          char unused;
          char aspect[10];
          int  signalnum, aspectbits;
          if (sscanf(buffer,"%c %d %9s",&unused,&signalnum,aspect) != 3) {
            Serial.println("\nSyntax error (Set command)!");
          } else {
            /* Parse aspect string. */
            aspectbits = GetAspectBits(aspect);
            /* Check for legal aspect string. */
            if (aspectbits < 0) {
              Serial.println("\nSyntax error (Bad aspect)!");
            /* Check for legal signal number. */
            } else if (signalnum >= 0 && signalnum < 8) {
              lc1.setRow(0, signalnum, (byte) aspectbits);
            } else {
              Serial.println("\nSyntax error (Bad signal number)!");
            }
          }
          break;
        }
      default:
         Serial.println("\nUnknown command!");
         break;  
    }
    /* Reissue command prompt. */
    Serial.print("\n>>");
    Serial.flush();
  }
}

